@@ -1,5 +1,8 @@
 """Core apispec classes and functions."""
-from collections import OrderedDict
+
+from __future__ import annotations
+
+import typing
 from copy import deepcopy
 import warnings
 
@@ -26,18 +29,26 @@ class Components:
     They became sub-fields of "components" top-level field in OAS v3.
     """
 
-    def __init__(self, plugins, openapi_version):
+    def __init__(
+        self,
+        plugins: tuple | list,
+        openapi_version: OpenAPIVersion,
+    ) -> None:
         self._plugins = plugins
         self.openapi_version = openapi_version
-        self.schemas = {}
-        self.responses = {}
-        self.parameters = {}
-        self.headers = {}
-        self.examples = {}
-        self.security_schemes = {}
+        self.schemas: dict[str, dict] = {}
+        self.responses: dict[str, dict] = {}
+        self.parameters: dict[str, dict] = {}
+        self.headers: dict[str, dict] = {}
+        self.examples: dict[str, dict] = {}
+        self.security_schemes: dict[str, dict] = {}
+        self.schemas_lazy: dict[str, dict] = {}
+        self.responses_lazy: dict[str, dict] = {}
+        self.parameters_lazy: dict[str, dict] = {}
+        self.headers_lazy: dict[str, dict] = {}
+        self.examples_lazy: dict[str, dict] = {}
 
-    def to_dict(self):
-        subsections = {
+        self._subsections = {
             "schema": self.schemas,
             "response": self.responses,
             "parameter": self.parameters,
@@ -45,17 +56,81 @@ class Components:
             "example": self.examples,
             "security_scheme": self.security_schemes,
         }
+        self._subsections_lazy = {
+            "schema": self.schemas_lazy,
+            "response": self.responses_lazy,
+            "parameter": self.parameters_lazy,
+            "header": self.headers_lazy,
+            "example": self.examples_lazy,
+        }
+
+    def to_dict(self) -> dict[str, dict]:
         return {
             COMPONENT_SUBSECTIONS[self.openapi_version.major][k]: v
-            for k, v in subsections.items()
+            for k, v in self._subsections.items()
             if v != {}
         }
 
-    def schema(self, component_id, component=None, **kwargs):
+    def _register_component(
+        self,
+        obj_type: str,
+        component_id: str,
+        component: dict,
+        *,
+        lazy: bool = False,
+    ) -> None:
+        subsection = (self._subsections if lazy is False else self._subsections_lazy)[
+            obj_type
+        ]
+        subsection[component_id] = component
+
+    def _do_register_lazy_component(
+        self,
+        obj_type: str,
+        component_id: str,
+    ) -> None:
+        component_buffer = self._subsections_lazy[obj_type]
+        # If component was lazy registered, register it for real
+        if component_id in component_buffer:
+            self._subsections[obj_type][component_id] = component_buffer.pop(
+                component_id
+            )
+
+    def get_ref(
+        self,
+        obj_type: str,
+        obj_or_component_id: dict | str,
+    ) -> dict:
+        """Return object or reference
+
+        If obj is a dict, it is assumed to be a complete description and it is returned as is.
+        Otherwise, it is assumed to be a reference name as string and the corresponding $ref
+        string is returned.
+
+        :param str subsection: "schema", "parameter", "response" or "security_scheme"
+        :param dict|str obj: object in dict form or as ref_id string
+        """
+        if isinstance(obj_or_component_id, dict):
+            return obj_or_component_id
+        # Register the component if it was lazy registered
+        self._do_register_lazy_component(obj_type, obj_or_component_id)
+        return build_reference(
+            obj_type, self.openapi_version.major, obj_or_component_id
+        )
+
+    def schema(
+        self,
+        component_id: str,
+        component: dict | None = None,
+        *,
+        lazy: bool = False,
+        **kwargs: typing.Any,
+    ) -> Components:
         """Add a new schema to the spec.
 
-        :param str component_id: identifier by which schema may be referenced.
-        :param dict component: schema definition.
+        :param str component_id: identifier by which schema may be referenced
+        :param dict component: schema definition
+        :param bool lazy: register component only when referenced in the spec
         :param kwargs: plugin-specific arguments
 
         .. note::
@@ -79,57 +154,69 @@ class Components:
             raise DuplicateComponentNameError(
                 f'Another schema with name "{component_id}" is already registered.'
             )
-        component = component or {}
-        ret = component.copy()
+        ret = deepcopy(component) or {}
         # Execute all helpers from plugins
         for plugin in self._plugins:
             try:
-                ret.update(
-                    plugin.schema_helper(component_id, component, **kwargs) or {}
-                )
+                ret.update(plugin.schema_helper(component_id, ret, **kwargs) or {})
             except PluginMethodNotImplementedError:
                 continue
         self._resolve_refs_in_schema(ret)
-        self.schemas[component_id] = ret
+        self._register_component("schema", component_id, ret, lazy=lazy)
         return self
 
-    def response(self, component_id, component=None, **kwargs):
+    def response(
+        self,
+        component_id: str,
+        component: dict | None = None,
+        *,
+        lazy: bool = False,
+        **kwargs: typing.Any,
+    ) -> Components:
         """Add a response which can be referenced.
 
         :param str component_id: ref_id to use as reference
         :param dict component: response fields
+        :param bool lazy: register component only when referenced in the spec
         :param kwargs: plugin-specific arguments
         """
         if component_id in self.responses:
             raise DuplicateComponentNameError(
                 f'Another response with name "{component_id}" is already registered.'
             )
-        component = component or {}
-        ret = component.copy()
+        ret = deepcopy(component) or {}
         # Execute all helpers from plugins
         for plugin in self._plugins:
             try:
-                ret.update(plugin.response_helper(component, **kwargs) or {})
+                ret.update(plugin.response_helper(ret, **kwargs) or {})
             except PluginMethodNotImplementedError:
                 continue
         self._resolve_refs_in_response(ret)
-        self.responses[component_id] = ret
+        self._register_component("response", component_id, ret, lazy=lazy)
         return self
 
-    def parameter(self, component_id, location, component=None, **kwargs):
+    def parameter(
+        self,
+        component_id: str,
+        location: str,
+        component: dict | None = None,
+        *,
+        lazy: bool = False,
+        **kwargs: typing.Any,
+    ) -> Components:
         """Add a parameter which can be referenced.
 
-        :param str component_id: identifier by which parameter may be referenced.
-        :param str location: location of the parameter.
-        :param dict component: parameter fields.
+        :param str component_id: identifier by which parameter may be referenced
+        :param str location: location of the parameter
+        :param dict component: parameter fields
+        :param bool lazy: register component only when referenced in the spec
         :param kwargs: plugin-specific arguments
         """
         if component_id in self.parameters:
             raise DuplicateComponentNameError(
                 f'Another parameter with name "{component_id}" is already registered.'
             )
-        component = component or {}
-        ret = component.copy()
+        ret = deepcopy(component) or {}
         ret.setdefault("name", component_id)
         ret["in"] = location
 
@@ -140,34 +227,53 @@ class Components:
         # Execute all helpers from plugins
         for plugin in self._plugins:
             try:
-                ret.update(plugin.parameter_helper(component, **kwargs) or {})
+                ret.update(plugin.parameter_helper(ret, **kwargs) or {})
             except PluginMethodNotImplementedError:
                 continue
         self._resolve_refs_in_parameter_or_header(ret)
-        self.parameters[component_id] = ret
+        self._register_component("parameter", component_id, ret, lazy=lazy)
         return self
 
-    def header(self, component_id, component):
+    def header(
+        self,
+        component_id: str,
+        component: dict,
+        *,
+        lazy: bool = False,
+        **kwargs: typing.Any,
+    ) -> Components:
         """Add a header which can be referenced.
 
-        :param str component_id: identifier by which header may be referenced.
-        :param dict component: header fields.
+        :param str component_id: identifier by which header may be referenced
+        :param dict component: header fields
+        :param bool lazy: register component only when referenced in the spec
+        :param kwargs: plugin-specific arguments
 
         https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.1.md#headerObject
         """
+        ret = deepcopy(component) or {}
         if component_id in self.headers:
             raise DuplicateComponentNameError(
                 f'Another header with name "{component_id}" is already registered.'
             )
-        self._resolve_refs_in_parameter_or_header(component)
-        self.headers[component_id] = component
+        # Execute all helpers from plugins
+        for plugin in self._plugins:
+            try:
+                ret.update(plugin.header_helper(ret, **kwargs) or {})
+            except PluginMethodNotImplementedError:
+                continue
+        self._resolve_refs_in_parameter_or_header(ret)
+        self._register_component("header", component_id, ret, lazy=lazy)
         return self
 
-    def example(self, component_id, component):
+    def example(
+        self, component_id: str, component: dict, *, lazy: bool = False
+    ) -> Components:
         """Add an example which can be referenced
 
-        :param str component_id: identifier by which example may be referenced.
-        :param dict component: example fields.
+        :param str component_id: identifier by which example may be referenced
+        :param dict component: example fields
+        :param bool lazy: register component only when referenced in the spec
 
         https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.1.md#exampleObject
         """
@@ -175,10 +281,10 @@ class Components:
             raise DuplicateComponentNameError(
                 f'Another example with name "{component_id}" is already registered.'
             )
-        self.examples[component_id] = component
+        self._register_component("example", component_id, component, lazy=lazy)
         return self
 
-    def security_scheme(self, component_id, component):
+    def security_scheme(self, component_id: str, component: dict) -> Components:
         """Add a security scheme which can be referenced.
 
         :param str component_id: component_id to use as reference
@@ -188,24 +294,10 @@ class Components:
             raise DuplicateComponentNameError(
                 f'Another security scheme with name "{component_id}" is already registered.'
             )
-        self.security_schemes[component_id] = component
+        self._register_component("security_scheme", component_id, component)
         return self
 
-    def get_ref(self, obj_type, obj):
-        """Return object or reference
-
-        If obj is a dict, it is assumed to be a complete description and it is returned as is.
-        Otherwise, it is assumed to be a reference name as string and the corresponding $ref
-        string is returned.
-
-        :param str obj_type: "schema", "parameter", "response" or "security_scheme"
-        :param dict|str obj: object in dict form or as ref_id string
-        """
-        if isinstance(obj, dict):
-            return obj
-        return build_reference(obj_type, self.openapi_version.major, obj)
-
-    def _resolve_schema(self, obj):
+    def _resolve_schema(self, obj) -> None:
         """Replace schema reference as string with a $ref if needed
 
         Also resolve references in the schema
@@ -214,12 +306,12 @@ class Components:
             obj["schema"] = self.get_ref("schema", obj["schema"])
             self._resolve_refs_in_schema(obj["schema"])
 
-    def _resolve_examples(self, obj):
+    def _resolve_examples(self, obj) -> None:
         """Replace example reference as string with a $ref"""
         for name, example in obj.get("examples", {}).items():
             obj["examples"][name] = self.get_ref("example", example)
 
-    def _resolve_refs_in_schema(self, schema):
+    def _resolve_refs_in_schema(self, schema: dict) -> None:
         if "properties" in schema:
             for key in schema["properties"]:
                 schema["properties"][key] = self.get_ref(
@@ -238,17 +330,17 @@ class Components:
             schema["not"] = self.get_ref("schema", schema["not"])
             self._resolve_refs_in_schema(schema["not"])
 
-    def _resolve_refs_in_parameter_or_header(self, parameter_or_header):
+    def _resolve_refs_in_parameter_or_header(self, parameter_or_header) -> None:
         self._resolve_schema(parameter_or_header)
         self._resolve_examples(parameter_or_header)
 
-    def _resolve_refs_in_request_body(self, request_body):
+    def _resolve_refs_in_request_body(self, request_body) -> None:
         # requestBody is OpenAPI v3+
         for media_type in request_body["content"].values():
             self._resolve_schema(media_type)
             self._resolve_examples(media_type)
 
-    def _resolve_refs_in_response(self, response):
+    def _resolve_refs_in_response(self, response) -> None:
         if self.openapi_version.major < 3:
             self._resolve_schema(response)
         else:
@@ -260,7 +352,7 @@ class Components:
                 self._resolve_refs_in_parameter_or_header(response["headers"][name])
             # TODO: Resolve link refs when Components supports links
 
-    def _resolve_refs_in_operation(self, operation):
+    def _resolve_refs_in_operation(self, operation) -> None:
         if "parameters" in operation:
             parameters = []
             for parameter in operation["parameters"]:
@@ -271,14 +363,14 @@ class Components:
         if "requestBody" in operation:
             self._resolve_refs_in_request_body(operation["requestBody"])
         if "responses" in operation:
-            responses = OrderedDict()
+            responses = {}
             for code, response in operation["responses"].items():
                 response = self.get_ref("response", response)
                 self._resolve_refs_in_response(response)
                 responses[code] = response
             operation["responses"] = responses
 
-    def resolve_refs_in_path(self, path):
+    def resolve_refs_in_path(self, path) -> None:
         if "parameters" in path:
             parameters = []
             for parameter in path["parameters"]:
@@ -313,7 +405,14 @@ class APISpec:
         See https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.2.md#openapi-object
     """
 
-    def __init__(self, title, version, openapi_version, plugins=(), **options):
+    def __init__(
+        self,
+        title: str,
+        version: str,
+        openapi_version: OpenAPIVersion | str,
+        plugins: tuple | list = (),
+        **options: typing.Any,
+    ) -> None:
         self.title = title
         self.version = version
         self.openapi_version = OpenAPIVersion(openapi_version)
@@ -321,8 +420,8 @@ class APISpec:
         self.plugins = plugins
 
         # Metadata
-        self._tags = []
-        self._paths = OrderedDict()
+        self._tags: list[dict] = []
+        self._paths: dict = {}
 
         # Components
         self.components = Components(self.plugins, self.openapi_version)
@@ -331,8 +430,8 @@ class APISpec:
         for plugin in self.plugins:
             plugin.init_spec(self)
 
-    def to_dict(self):
-        ret = {
+    def to_dict(self) -> dict[str, typing.Any]:
+        ret: dict[str, typing.Any] = {
             "paths": self._paths,
             "info": {"title": self.title, "version": self.version},
         }
@@ -349,7 +448,7 @@ class APISpec:
         ret = deepupdate(ret, self.options)
         return ret
 
-    def to_yaml(self, yaml_dump_kwargs=None):
+    def to_yaml(self, yaml_dump_kwargs: typing.Any | None = None) -> str:
         """Render the spec to YAML. Requires PyYAML to be installed.
 
         :param dict yaml_dump_kwargs: Additional keyword arguments to pass to `yaml.dump`
@@ -358,7 +457,7 @@ class APISpec:
 
         return dict_to_yaml(self.to_dict(), yaml_dump_kwargs)
 
-    def tag(self, tag):
+    def tag(self, tag: dict) -> APISpec:
         """Store information about a tag.
 
         :param dict tag: the dictionary storing information about the tag.
@@ -368,14 +467,14 @@ class APISpec:
 
     def path(
         self,
-        path=None,
+        path: dict | None = None,
         *,
-        operations=None,
-        summary=None,
-        description=None,
-        parameters=None,
-        **kwargs,
-    ):
+        operations: dict[str, typing.Any] | None = None,
+        summary: str | None = None,
+        description: str | None = None,
+        parameters: list[dict] | None = None,
+        **kwargs: typing.Any,
+    ) -> APISpec:
         """Add a new path object to the spec.
 
         https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.2.md#path-item-object
@@ -389,7 +488,7 @@ class APISpec:
         """
         # operations and parameters must be deepcopied because they are mutated
         # in _clean_operations and operation helpers and path may be called twice
-        operations = deepcopy(operations) or OrderedDict()
+        operations = deepcopy(operations) or {}
         parameters = deepcopy(parameters) or []
 
         # Execute path helpers
@@ -427,7 +526,10 @@ class APISpec:
 
         return self
 
-    def _clean_parameters(self, parameters):
+    def _clean_parameters(
+        self,
+        parameters: list[dict],
+    ) -> list[dict]:
         """Ensure that all parameters with "in" equal to "path" are also required
         as required by the OpenAPI specification, as well as normalizing any
         references to global parameters and checking for duplicates parameters
@@ -463,7 +565,10 @@ class APISpec:
 
         return parameters
 
-    def _clean_operations(self, operations):
+    def _clean_operations(
+        self,
+        operations: dict[str, dict],
+    ) -> None:
         """Ensure that all parameters with "in" equal to "path" are also required
         as required by the OpenAPI specification, as well as normalizing any
         references to global parameters. Also checks for invalid HTTP methods.
@@ -488,7 +593,7 @@ class APISpec:
                     operation["parameters"]
                 )
             if "responses" in operation:
-                responses = OrderedDict()
+                responses = {}
                 for code, response in operation["responses"].items():
                     try:
                         code = int(code)  # handles IntEnums like http.HTTPStatus
