@@ -1,25 +1,20 @@
 """YAML utilities"""
 
-from collections import OrderedDict
+from __future__ import annotations
+
 import yaml
+import typing
 
 from apispec.utils import trim_docstring, dedent
 
 
-class YAMLDumper(yaml.Dumper):
-    @staticmethod
-    def _represent_dict(dumper, instance):
-        return dumper.represent_mapping("tag:yaml.org,2002:map", instance.items())
+def dict_to_yaml(dic: dict, yaml_dump_kwargs: typing.Any | None = None) -> str:
+    if yaml_dump_kwargs is None:
+        yaml_dump_kwargs = {}
+    return yaml.dump(dic, **yaml_dump_kwargs)
 
 
-yaml.add_representer(OrderedDict, YAMLDumper._represent_dict, Dumper=YAMLDumper)
-
-
-def dict_to_yaml(dic):
-    return yaml.dump(dic, Dumper=YAMLDumper)
-
-
-def load_yaml_from_docstring(docstring):
+def load_yaml_from_docstring(docstring: str) -> dict:
     """Loads YAML from docstring."""
     split_lines = trim_docstring(docstring).split("\n")
 
@@ -40,7 +35,7 @@ def load_yaml_from_docstring(docstring):
 PATH_KEYS = {"get", "put", "post", "delete", "options", "head", "patch"}
 
 
-def load_operations_from_docstring(docstring):
+def load_operations_from_docstring(docstring: str) -> dict:
     """Return a dictionary of OpenAPI operations parsed from a
     a docstring.
     """
